import os
import time
import axipy
from PySide2 import QtCore
from PySide2.QtCore import QFile, QThread, Qt
from PySide2.QtGui import QDoubleValidator
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QApplication, QDialog
from axipy import data_manager, SelectionTable, Notifications, LinearUnit

from Undershot.ui.process.overShotProcess import ProcessUnderOverShot,ProcessOverShot,  ProcessPrepareDelOverShot
from Undershot.ui.process.underShotProcess import ProcessUnderShot


class DlgUnderOverShot(QDialog):
    cancelProcess = QtCore.Signal()
    def __init__(self,parent=None):
        self.__parentWin=parent
        super(DlgUnderOverShot, self).__init__(parent)
        self.load_ui("DlgUnder_over_shot.ui")
        self.__initLinerUnits()
        validator_tolernce=QDoubleValidator()
        validator_tolernce.setBottom(0.0)
        validator_tolernce.setNotation(QDoubleValidator.StandardNotation)
        self.ui.ln_tolerant.setValidator(validator_tolernce)
        self.ui.ln_tolerant.textChanged.connect(self.__change_tolerance)
        self.ui.ch_over.stateChanged.connect(self.__setEnableRun)
        self.ui.ch_under.stateChanged.connect(self.__setEnableRun)
        self.ui.cmb_tables.currentIndexChanged.connect(self.__change_tab)
        self.ui.pb_run.clicked.connect(self.__run)
        self.ui.pb_cancel.clicked.connect(self.__cancel)
        self.ui.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
        self.ui.setWindowFlag(Qt.WindowCloseButtonHint, False)
        self.__isRun = False
        self.__process=None
    def load_ui(self, name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__), name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.ui = loader.load(ui_file, self.__parentWin)

        ui_file.close()
    def __initLinerUnits(self):
        units=axipy.LinearUnits.values()
        for item in units:
            if item==LinearUnit.m:
                self.ui.cmb_line_units.addItem(item.localized_name)
    def __change_tolerance(self,valur_str):
        db_tolerance = float(valur_str)
        if db_tolerance <= 0.0:
            self.ui.pb_run.setEnabled(False)
            Notifications.push('Удаление переводов недоводов', 'Толерантность должна быть >0', Notifications.Warning)
    def __change_tab(self,index_tab):
        tab_sel=data_manager.find(self.ui.cmb_tables.currentText())
        name_unit=tab_sel.coordsystem.unit.localized_name
        for i in range(self.ui.cmb_line_units.count()):
            if self.ui.cmb_line_units.itemText(i)==name_unit:
                self.ui.cmb_line_units.setCurrentIndex(i)
                break

    def __updateTable(self):
        self.ui.cmb_tables.clear()
        for tab in data_manager.tables:
            '''
            if isinstance(tab,SelectionTable) :
                continue
            '''
            if (not tab.is_spatial) or ( not tab.is_editable):
                continue
            if tab.coordsystem.lat_lon:
                continue
            if isinstance(tab,SelectionTable) :
                self.ui.cmb_tables.insertItem(0,tab.name)
            else:
                self.ui.cmb_tables.addItem(tab.name)
        self.ui.cmb_tables.setCurrentIndex(0)
    def __setEnableRun(self):
        db_tolerance=float(self.ui.ln_tolerant.text())
        if db_tolerance<=0.0:
            self.ui.pb_run.setEnabled(False)
            Notifications.push('Удаление переводов недоводов', 'Толерантность должна быть >0', Notifications.Warning)
        if self.ui.ch_over.isEnabled() or self.ui.ch_under.isEnabled():
            self.ui.pb_run.setEnabled(True)
        else:
            self.ui.pb_run.setEnabled(False)
    def update_progress(self,value_main):

        self.ui.prog_bar.setValue(value_main)

            # print(value)
        time.sleep(0.0001)
        QApplication.processEvents()
    def setFormatprogressBar(self, str_format_main):
        self.ui.prog_bar.setFormat(str_format_main)

        import time
        time.sleep(0.0001)
        QApplication.processEvents()
    def __run(self):
        self.__isRun=True
        list_process=[]
        self.ui.pb_run.setEnabled(False)
        #self.ui.pb_cancel.setEnabled(True)
        self.ui.pb_cancel.setText("Прервать")
        params={}
        params['table']=self.ui.cmb_tables.currentText()
        params['tolerance']=float(self.ui.ln_tolerant.text())
        params['unit']=self.ui.cmb_line_units.currentText()
        params['delOvetShot']=True
        isOverShot=self.ui.ch_over.isChecked()
        isUnderShot=self.ui.ch_under.isChecked()
        if isUnderShot and isOverShot:
            list_process.append(ProcessOverShot(params,self))
            list_process.append(ProcessPrepareDelOverShot(params,self))
            list_process.append(ProcessUnderShot(params,self))
        if isOverShot and (not isUnderShot):
            list_process.append(ProcessOverShot(params, self))
            list_process.append(ProcessPrepareDelOverShot(params, self))
        if (not isOverShot) and isUnderShot:
            params['delOvetShot'] = False
            list_process.append(ProcessPrepareDelOverShot(params, self))
            list_process.append(ProcessUnderShot(params, self))
        self.__process=ProcessUnderOverShot({'process':list_process},self)
        self.__process.finished.connect(self.__end_process)
        self.thread = QThread()
        self.__process.moveToThread(self.thread)
        #self.thread.started.connect(self.__process.run)
        self.__process.run()
        #self.thread.start()
    def __cancel(self):
        if not self.__isRun:
            self.ui.close()
            return
        if self.__process is not None:
            self.__process_status = False
            self.cancelProcess.emit()
            #self.__process.setCancel()
    def __end_process(self, value):
        self.__isRun=False
        self.ui.pb_run.setEnabled(True)
        #self.ui.pb_cancel.setEnabled(False)
        self.ui.pb_cancel.setText("Закрыть")
        self.__process.finished.disconnect(self.__end_process)
        # self.__process.disconnect(self.__end_process)

        self.__process = None
        self.thread = None
    @property
    def widget(self):
        return self.ui

    def show(self):
        # self.ui.showMaximized()
        # print("Show Form")
        self.__updateTable()
        self.ui.show()